/**
 * Copyright © 2013-2017 Magento, Inc. All rights reserved.
 * See COPYING.txt for license details.
 */

define([
    'underscore',
    'Magento_Ui/js/form/element/abstract',
    'mageUtils',
    'Magento_Ui/js/lib/validation/validator',
    'jquery',
    'jquery/ui',
    'jquery/validate',
    'mage/translate',
    'intlTelInput'
], function (_, Abstract, utils, validator, $) {
    'use strict';

    return Abstract.extend({
        /**
         * Handler function which is supposed to be invoked when
         * file input element has been rendered.
         *
         * @param {HTMLInputElement} fileInput
         */
        onElementRender: function (input) {
            if (window.checkoutConfig.intlTelInput) {
                var currentInput = $(input);

                if (window.checkoutConfig.intlTelInput.geolocation) {
                    currentInput.intlTelInput({
                        initialCountry: "auto",
                        geoIpLookup: function (callback) {
                            $.get('https://ipinfo.io', function () {}, "jsonp").always(function (resp) {
                                var countryCode = (resp && resp.country) ? resp.country : "";
                                callback(countryCode);
                            });
                        },
                        utilsScript: window.checkoutConfig.intlTelInput.utilsScriptUrl
                    });
                } else {
                    currentInput.intlTelInput({
                        utilsScript: window.checkoutConfig.intlTelInput.utilsScriptUrl
                    });
                }

                if (window.checkoutConfig.intlTelInput.validation) {

                    currentInput.blur(function () {
                        var that = $(this);

                        if ($.trim(that.val())) {
                            that.val(that.intlTelInput("getNumber")).change();

                        }
                    });
                }
            }
        },

        validate: function () {
            var value = this.value(),
                    result = validator(this.validation, value, this.validationParams),
                    message = !this.disabled() && this.visible() ? result.message : '',
                    isValid = this.disabled() || !this.visible() || result.passed;

            this.error(message);
            this.bubble('error', message);

            //TODO: Implement proper result propagation for form
            if (!isValid) {
                this.source.set('params.invalid', true);
            }

            if (isValid && window.checkoutConfig.intlTelInput.validation) {
                var inp = $("<input type='tel' value='" + value + "' />");
                inp.intlTelInput({
                    utilsScript: window.checkoutConfig.intlTelInput.utilsScriptUrl
                });
                isValid = this.disabled() || !this.visible() || inp.intlTelInput("isValidNumber");

                if (!isValid) {
                    message = $.mage.__('Please enter a valid number');
                    this.error(message);
                    this.bubble('error', message);
                    this.source.set('params.invalid', true);
                }
            }

            return {
                valid: isValid,
                target: this
            };
        },

        initConfig: function () {

            var uid = utils.uniqueid(),
                    name,
                    valueUpdate,
                    scope;

            this._super();

            scope = this.dataScope;
            name = scope.split('.').slice(1);

            valueUpdate = this.showFallbackReset ? 'afterkeydown' : this.valueUpdate;

            _.extend(this, {
                uid: uid,
                noticeId: 'notice-' + uid,
                inputName: utils.serializeName(name.join('.')),
                valueUpdate: valueUpdate,
                onElementRender: this.onElementRender
            });

            return this;
        }
    });
});
